# src/tab_dynamic.py

import tkinter as tk
from tkinter import ttk
from tkinter.scrolledtext import ScrolledText
import math
import numpy as np

MU0 = 4.0e-7 * math.pi

class TabDynamic:
    def __init__(self, notebook, tab_coil=None, tab_simulator=None, tab_drag=None):
        """
        Pestaña que:
         - Obtiene el trabajo W desde tab_simulator.
         - Calcula la reluctancia media con la integración en x in [0, h_c].
         - Usa el modelo W = ((N I)^2 * h_c)/(4 mu0 S_fe (sumR_avg)^2) para hallar I.
         - Ofrece dimensionamiento de fuente (condensador / DC).
        """
        self.notebook = notebook
        self.tab_coil = tab_coil
        self.tab_simulator = tab_simulator
        self.tab_drag = tab_drag

        self.frame = tk.Frame(notebook)
        self.frame.pack(fill="both", expand=True)

        # Panel Izq: Botones / parámetros
        left_frame = tk.Frame(self.frame)
        left_frame.pack(side="left", fill="y", padx=5, pady=5)

        # Panel Der: Logs
        right_frame = tk.Frame(self.frame, bd=2, relief="groove")
        right_frame.pack(side="right", fill="both", expand=True, padx=5, pady=5)

        # Sección 1: Importar Config
        import_frame = tk.LabelFrame(left_frame, text="1) Importar Config")
        import_frame.pack(fill="x", pady=5)

        btn_import = tk.Button(import_frame, text="Importar Config", command=self.importar_config)
        btn_import.pack(pady=5)

        self.var_W_mech = tk.StringVar(value="")
        tk.Label(import_frame, text="W (J):").pack(anchor="w")
        tk.Entry(import_frame, textvariable=self.var_W_mech, width=12, state="readonly").pack(pady=2)

        self.W_mech = 0.0  # energía mecánica importada

        # Sección 2: Calcular Reluctancia Media
        rel_frame = tk.LabelFrame(left_frame, text="2) Reluctancia Media")
        rel_frame.pack(fill="x", pady=5)

        self.var_sumR_avg = tk.StringVar(value="")
        tk.Button(rel_frame, text="Calcular Rel. Media", command=self.calcular_reluctancia_media).pack(pady=5)
        tk.Label(rel_frame, text="sum(R)_avg:").pack(anchor="w")
        tk.Entry(rel_frame, textvariable=self.var_sumR_avg, width=15, state="readonly").pack(pady=2)

        self.sumR_avg = 0.0  # valor en el interior

        # Sección 3: Calcular Corriente
        work_frame = tk.LabelFrame(left_frame, text="3) Corriente Necesaria")
        work_frame.pack(fill="x", pady=5)

        self.var_S_fe   = tk.DoubleVar(value=1e-4)  # Sección en la zona férrea
        tk.Label(work_frame, text="S_fe (m²):").grid(row=0, column=0, sticky="w", padx=5, pady=2)
        tk.Entry(work_frame, textvariable=self.var_S_fe, width=10).grid(row=0, column=1, padx=5, pady=2)

        tk.Button(work_frame, text="Calcular Corriente", command=self.calcular_corriente_trabajo).grid(row=1, column=0, columnspan=2, pady=5)

        self.var_I_req = tk.StringVar(value="")
        tk.Label(work_frame, text="I_req (A):").grid(row=2, column=0, sticky="w", padx=5, pady=2)
        tk.Entry(work_frame, textvariable=self.var_I_req, width=10, state="readonly").grid(row=2, column=1, padx=5, pady=2)

        # Sección 4: Dimensionar Fuente
        fuente_frame = tk.LabelFrame(left_frame, text="4) Dimensionar Fuente")
        fuente_frame.pack(fill="x", pady=5)

        tk.Button(fuente_frame, text="Dimensionar Capacitor", command=self.dimensionar_condensador).pack(pady=5)
        tk.Button(fuente_frame, text="Fuente DC minima", command=self.dimensionar_fuente_dc).pack(pady=5)

        self.var_C_sug   = tk.StringVar(value="")
        self.var_V_sug   = tk.StringVar(value="")
        self.var_Vdc_min = tk.StringVar(value="")

        tk.Label(fuente_frame, text="C (F) ~").pack(anchor="w")
        tk.Entry(fuente_frame, textvariable=self.var_C_sug, width=15, state="readonly").pack(pady=2)

        tk.Label(fuente_frame, text="Vcap (V) ~").pack(anchor="w")
        tk.Entry(fuente_frame, textvariable=self.var_V_sug, width=15, state="readonly").pack(pady=2)

        tk.Label(fuente_frame, text="V_DC min (V) ~").pack(anchor="w")
        tk.Entry(fuente_frame, textvariable=self.var_Vdc_min, width=15, state="readonly").pack(pady=2)

        # LOGS
        tk.Label(right_frame, text="Logs:").pack(anchor="w")
        self.log_area = ScrolledText(right_frame, wrap="word", height=25, width=50)
        self.log_area.pack(fill="both", expand=True)
        self.log_area.configure(state="disabled")
        self.log_area.tag_config("info",  foreground="green")
        self.log_area.tag_config("error", foreground="red")

    # --------------------------------------------------------
    # Log helper
    # --------------------------------------------------------
    def log_message(self, msg, mode="info"):
        self.log_area.configure(state="normal")
        self.log_area.insert("end", msg + "\n", mode)
        self.log_area.see("end")
        self.log_area.configure(state="disabled")

    # --------------------------------------------------------
    # 1) Importar Config (energía W)
    # --------------------------------------------------------
    def importar_config(self):
        """
        Lee la energía mecánica W de tab_simulator y la guarda en self.W_mech
        """
        try:
            if not self.tab_simulator:
                raise ValueError("No hay tab_simulator para leer W.")
            w_val = self.tab_simulator.get_energy_required()
            if w_val<=0:
                raise ValueError("Energía <= 0.")
            self.W_mech = w_val
            self.var_W_mech.set(f"{w_val:.3f}")
            self.log_message(f"Importado W={w_val:.3f} J", "info")

        except Exception as e:
            self.log_message(f"Error importar_config: {e}", "error")

    # --------------------------------------------------------
    # 2) Calcular Reluctancia Media (sum(R)_avg)
    #    => integrando la ecuación sum_R(x)= ...
    # --------------------------------------------------------
    def calcular_reluctancia_media(self):
        try:
            if not self.tab_coil:
                raise ValueError("Falta referencia a tab_coil.")
            if not self.tab_drag:
                raise ValueError("Falta referencia a tab_drag.")

            # 1) Leer h_c, N, r_int (en mm->m)
            h_c_m = float(self.tab_coil.var_h_c.get())/1000.0
            if h_c_m<=0:
                raise ValueError("h_c_m <=0.")

            r_int_m = float(self.tab_coil.var_r_int.get())/1000.0
            if r_int_m<=0:
                raise ValueError("r_int_m <=0.")

            # 2) l_fe => de tab_drag
            l_fe_m = self.tab_drag.get_l_fe()  # en metros
            if l_fe_m<0:
                raise ValueError("l_fe<0?")

            # 3) s_disp => get_projectile_section_for_reluctance
            s_disp = self.tab_drag.get_projectile_section_for_reluctance(self.tab_coil)
            if s_disp<=0:
                raise ValueError("s_disp <=0.")

            # 4) s_c= pi*r_int^2
            s_c = math.pi*(r_int_m**2)
            if s_c<=0:
                raise ValueError("s_c<=0.")

            # 5) Definir sumR(x)
            def sumR_of_x(x):
                # sum_R = h_c/(mu0*s_disp)
                #       + (h_c + l_fe - x)/(mu0*s_disp)
                #       + (h_c - x)/(mu0*s_c)
                return (
                    (h_c_m/(MU0*s_disp))
                    + ((h_c_m + l_fe_m - x)/(MU0*s_disp))
                    + ((h_c_m - x)/(MU0*s_c))
                )

            # 6) Integración en [0..h_c_m]
            n_steps=200
            dx = h_c_m/n_steps
            area_sum=0.0
            x_curr=0.0
            for _ in range(n_steps):
                x_mid= x_curr+0.5*dx
                val_mid= sumR_of_x(x_mid)
                area_sum+= val_mid
                x_curr+=dx
            area_sum*= dx

            sumR_avg= area_sum/h_c_m
            if sumR_avg<=1e-30:
                raise ValueError("sumR_avg nulo o negativo.")

            self.sumR_avg= sumR_avg
            self.var_sumR_avg.set(f"{sumR_avg:.3e}")
            self.log_message(f"Reluctancia media= {sumR_avg:.3e} H^-1", "info")

        except ValueError as ve:
            self.sumR_avg=0.0
            self.var_sumR_avg.set("")
            self.log_message(f"Error calc Reluctancia media: {ve}", "error")
        except Exception as ex:
            self.sumR_avg=0.0
            self.var_sumR_avg.set("")
            self.log_message(f"Excepción calc Reluct. media: {ex}", "error")

    # --------------------------------------------------------
    # 3) Calcular Corriente, con W = ((N I)^2 * h_c)/(4 mu0 S_fe (sumR_avg)^2)
    # --------------------------------------------------------
    def calcular_corriente_trabajo(self):
        try:
            if self.W_mech<=0:
                raise ValueError("No hay W_mech positivo. Importa config primero.")
            if self.sumR_avg<=0:
                raise ValueError("sumR_avg<=0. Calcula la Reluctancia media antes.")

            # 1) Leer h_c (mm->m), N, etc. de tab_coil
            h_c_m = float(self.tab_coil.var_h_c.get())/1000.0
            N_val = float(self.tab_coil.var_N.get())
            if h_c_m<=0 or N_val<=0:
                raise ValueError("Parámetros de bobina inválidos.")

            # 2) Leer S_fe
            S_fe_val= self.var_S_fe.get()
            if S_fe_val<=0:
                raise ValueError("S_fe <=0")

            # 3) Tomamos sumR_avg, W
            sumR_avg= self.sumR_avg
            W= self.W_mech

            # 4) W= ((N I)^2 * h_c)/(4 mu0 S_fe (sumR_avg)^2)
            # => (N I)^2= [4 mu0 S_fe (sumR_avg)^2 W]/h_c
            top= 4.0*MU0*S_fe_val*(sumR_avg**2)* W
            bottom= h_c_m
            if bottom<=1e-30:
                raise ValueError("h_c_m=0?")

            NI_sq= top/bottom
            if NI_sq<=0:
                raise ValueError("Resultado (N I)^2 <=0, no válido.")

            NI= math.sqrt(NI_sq)
            I_req= NI/ N_val

            self.var_I_req.set(f"{I_req:.3f}")
            self.log_message(f"I necesaria= {I_req:.3f} A", "info")

        except ValueError as ve:
            self.var_I_req.set("")
            self.log_message(f"Error calc I trabajo: {ve}", "error")
        except Exception as ex:
            self.var_I_req.set("")
            self.log_message(f"Excepción calc I trabajo: {ex}", "error")

    # --------------------------------------------------------
    # 4) Dimensionar la fuente (ejemplo condensador y DC)
    # --------------------------------------------------------
    def dimensionar_condensador(self):
        """
        Ejemplo: suponer que la energía total W -> (1/2)C V^2
        W_marg = W_mech * factor... etc. 
        """
        try:
            W= self.W_mech
            if W<=0:
                raise ValueError("Primero importa config. W<=0?")

            # suponer un factor "pérdidas"
            eff=0.7
            # E capacitor= W/ eff
            E_cap= W/eff
            # asumes C= 1mF => V= sqrt(2 E_cap/C)
            # o V= 200 => C= 2E_cap/V^2, etc.
            # Aquí arbitrariamente p.ej C=1e-3 y calculamos V
            C_val= 1e-3
            Vcalc= math.sqrt(2.0*E_cap/C_val)

            self.var_C_sug.set(f"{C_val:.3g}")
            self.var_V_sug.set(f"{Vcalc:.1f}")
            self.log_message(f"Caps: C=1mF => V~{Vcalc:.1f} V (descarga)", "info")

        except ValueError as ve:
            self.log_message(f"Error dimensionar_condensador: {ve}", "error")
        except Exception as ex:
            self.log_message(f"Excepción dimensionar_condensador: {ex}", "error")

    def dimensionar_fuente_dc(self):
        """
        Ejemplo simple: V_dc >= R_coil * I_req 
        R_coil en ohms => la obtendrías o la metes manual. 
        """
        try:
            Ireq_str= self.var_I_req.get()
            Ireq= float(Ireq_str)
            if Ireq<=0:
                raise ValueError("I_req <=0? Calcula la corriente primero.")

            # supongamos R_coil= 2 ohms (ejemplo)
            R_coil= 2.0
            V_dc= R_coil* Ireq
            self.var_Vdc_min.set(f"{V_dc:.1f}")
            self.log_message(f"Fuente DC~ {V_dc:.1f} V (min. ohmico)", "info")

        except ValueError as ve:
            self.log_message(f"Error dimensionar_fuente_dc: {ve}", "error")
        except Exception as ex:
            self.log_message(f"Excepción dimensionar_fuente_dc: {ex}", "error")
